@@ -3,10 +3,10 @@
  *
  *	Functions maintaining the in-memory configuration information
  *
- *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: runtime_config.c,v 1.19.2.1 2004/09/30 17:45:06 cbbrowne Exp $
+ *	$Id: runtime_config.c,v 1.24.2.1 2006-01-06 17:07:47 cbbrowne Exp $
  *-------------------------------------------------------------------------
  */
 
@@ -29,35 +29,36 @@
 #include "slon.h"
 
 
-/* ----------
- * Global data
- * ----------
+/*
+ * ---------- Global data ----------
  */
-pid_t					slon_pid;
-char				   *rtcfg_cluster_name = NULL;
-char				   *rtcfg_namespace = NULL;
-char				   *rtcfg_conninfo = NULL;
-int						rtcfg_nodeid = -1;
-int						rtcfg_nodeactive = 0;
-char				   *rtcfg_nodecomment = NULL;
-char					rtcfg_lastevent[64];
+pid_t		slon_pid;
+pid_t		slon_cpid;
+pid_t		slon_ppid;
+char	   *rtcfg_cluster_name = NULL;
+char	   *rtcfg_namespace = NULL;
+char	   *rtcfg_conninfo = NULL;
+int			rtcfg_nodeid = -1;
+int			rtcfg_nodeactive = 0;
+char	   *rtcfg_nodecomment = NULL;
+char		rtcfg_lastevent[64];
 
-SlonSet				   *rtcfg_set_list_head = NULL;
-SlonSet				   *rtcfg_set_list_tail = NULL;
-SlonNode			   *rtcfg_node_list_head = NULL;
-SlonNode			   *rtcfg_node_list_tail = NULL;
+SlonSet    *rtcfg_set_list_head = NULL;
+SlonSet    *rtcfg_set_list_tail = NULL;
+SlonNode   *rtcfg_node_list_head = NULL;
+SlonNode   *rtcfg_node_list_tail = NULL;
 
 
-/* ----------
- * Local data
- * ----------
+/*
+ * ---------- Local data ----------
  */
-static pthread_mutex_t	config_lock = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t	cfgseq_lock = PTHREAD_MUTEX_INITIALIZER;
-static int64			cfgseq = 0;
+static pthread_mutex_t config_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t cfgseq_lock = PTHREAD_MUTEX_INITIALIZER;
+static int64 cfgseq = 0;
 
-struct to_activate {
-	int					no_id;
+struct to_activate
+{
+	int			no_id;
 
 	struct to_activate *prev;
 	struct to_activate *next;
@@ -66,16 +67,14 @@ static struct to_activate *to_activate_head = NULL;
 static struct to_activate *to_activate_tail = NULL;
 
 
-/* ----------
- * Local functions
- * ----------
+/*
+ * ---------- Local functions ----------
  */
-static void			rtcfg_startStopNodeThread(SlonNode *node);
+static void rtcfg_startStopNodeThread(SlonNode * node);
 
 
-/* ----------
- * rtcfg_lock
- * ----------
+/*
+ * ---------- rtcfg_lock ----------
  */
 void
 rtcfg_lock(void)
@@ -84,9 +83,8 @@ rtcfg_lock(void)
 }
 
 
-/* ----------
- * rtcfg_unlock
- * ----------
+/*
+ * ---------- rtcfg_unlock ----------
  */
 void
 rtcfg_unlock(void)
@@ -95,14 +93,13 @@ rtcfg_unlock(void)
 }
 
 
-/* ----------
- * rtcfg_storeNode
- * ----------
+/*
+ * ---------- rtcfg_storeNode ----------
  */
 void
 rtcfg_storeNode(int no_id, char *no_comment)
 {
-	SlonNode	   *node;
+	SlonNode   *node;
 
 	rtcfg_lock();
 
@@ -113,8 +110,8 @@ rtcfg_storeNode(int no_id, char *no_comment)
 	if (node)
 	{
 		slon_log(SLON_CONFIG,
-				"storeNode: no_id=%d no_comment='%s' - update node\n", 
-				no_id, no_comment);
+				 "storeNode: no_id=%d no_comment='%s' - update node\n",
+				 no_id, no_comment);
 
 		free(node->no_comment);
 		node->no_comment = strdup(no_comment);
@@ -127,19 +124,19 @@ rtcfg_storeNode(int no_id, char *no_comment)
 	 * Add the new node to our in-memory configuration.
 	 */
 	slon_log(SLON_CONFIG,
-			"storeNode: no_id=%d no_comment='%s'\n",
-			no_id, no_comment);
+			 "storeNode: no_id=%d no_comment='%s'\n",
+			 no_id, no_comment);
 
-	node = (SlonNode *)malloc(sizeof(SlonNode));
+	node = (SlonNode *) malloc(sizeof(SlonNode));
 	if (node == NULL)
 	{
 		perror("rtcfg_storeNode: malloc()");
 		slon_abort();
 	}
-	memset (node, 0, sizeof(SlonNode));
+	memset(node, 0, sizeof(SlonNode));
 
-	node->no_id      = no_id;
-	node->no_active  = false;
+	node->no_id = no_id;
+	node->no_active = false;
 	node->no_comment = strdup(no_comment);
 	pthread_mutex_init(&(node->message_lock), NULL);
 	pthread_cond_init(&(node->message_cond), NULL);
@@ -151,15 +148,13 @@ rtcfg_storeNode(int no_id, char *no_comment)
 }
 
 
-/* ----------
- * rtcfg_setNodeLastEvent()
+/*
+ * ---------- rtcfg_setNodeLastEvent()
  *
- *	Set the last_event field in the node runtime structure.
+ * Set the last_event field in the node runtime structure.
  *
- *	Returns:	0 if the event_seq is <= the known value
- *				-1 if the node is not known
- *				event_seq otherwise
- * ----------
+ * Returns: 0 if the event_seq is <= the known value -1 if the node is
+ * not known event_seq otherwise ----------
  */
 int64
 rtcfg_setNodeLastEvent(int no_id, int64 event_seq)
@@ -184,18 +179,17 @@ rtcfg_setNodeLastEvent(int no_id, int64 event_seq)
 	rtcfg_unlock();
 
 	slon_log(SLON_DEBUG2,
-			"setNodeLastEvent: no_id=%d event_seq=" INT64_FORMAT "\n",
-			no_id, retval);
+			 "setNodeLastEvent: no_id=%d event_seq=" INT64_FORMAT "\n",
+			 no_id, retval);
 
 	return retval;
 }
 
 
-/* ----------
- * rtcfg_getNodeLastEvent
+/*
+ * ---------- rtcfg_getNodeLastEvent
  *
- *	Read the nodes last_event field
- * ----------
+ * Read the nodes last_event field ----------
  */
 int64
 rtcfg_getNodeLastEvent(int no_id)
@@ -217,14 +211,13 @@ rtcfg_getNodeLastEvent(int no_id)
 }
 
 
-/* ----------
- * rtcfg_enableNode
- * ----------
+/*
+ * ---------- rtcfg_enableNode ----------
  */
 void
 rtcfg_enableNode(int no_id)
 {
-	SlonNode	   *node;
+	SlonNode   *node;
 
 	rtcfg_lock();
 
@@ -234,7 +227,7 @@ rtcfg_enableNode(int no_id)
 		rtcfg_unlock();
 
 		slon_log(SLON_FATAL,
-				"enableNode: unknown node ID %d\n", no_id);
+				 "enableNode: unknown node ID %d\n", no_id);
 		slon_abort();
 		return;
 	}
@@ -243,7 +236,7 @@ rtcfg_enableNode(int no_id)
 	 * Activate the node
 	 */
 	slon_log(SLON_CONFIG,
-			"enableNode: no_id=%d\n", no_id);
+			 "enableNode: no_id=%d\n", no_id);
 	node->no_active = true;
 
 	rtcfg_unlock();
@@ -253,14 +246,13 @@ rtcfg_enableNode(int no_id)
 }
 
 
-/* ----------
- * slon_disableNode
- * ----------
+/*
+ * ---------- slon_disableNode ----------
  */
 void
 rtcfg_disableNode(int no_id)
 {
-	SlonNode	   *node;
+	SlonNode   *node;
 
 	rtcfg_lock();
 
@@ -270,7 +262,7 @@ rtcfg_disableNode(int no_id)
 		rtcfg_unlock();
 
 		slon_log(SLON_FATAL,
-				"enableNode: unknown node ID %d\n", no_id);
+				 "enableNode: unknown node ID %d\n", no_id);
 		slon_abort();
 		return;
 	}
@@ -279,26 +271,25 @@ rtcfg_disableNode(int no_id)
 	 * Deactivate the node
 	 */
 	slon_log(SLON_CONFIG,
-			"disableNode: no_id=%d\n", no_id);
+			 "disableNode: no_id=%d\n", no_id);
 	node->no_active = false;
 
 	rtcfg_unlock();
 	rtcfg_seq_bump();
 
 	/*
-	rtcfg_startStopNodeThread(node);
-	*/
+	 * rtcfg_startStopNodeThread(node);
+	 */
 }
 
 
-/* ----------
- * rtcfg_findNode
- * ----------
+/*
+ * ---------- rtcfg_findNode ----------
  */
 SlonNode *
 rtcfg_findNode(int no_id)
 {
-	SlonNode	   *node;
+	SlonNode   *node;
 
 	for (node = rtcfg_node_list_head; node; node = node->next)
 	{
@@ -310,14 +301,13 @@ rtcfg_findNode(int no_id)
 }
 
 
-/* ----------
- * rtcfg_storePath
- * ----------
+/*
+ * ---------- rtcfg_storePath ----------
  */
 void
 rtcfg_storePath(int pa_server, char *pa_conninfo, int pa_connretry)
 {
-	SlonNode	   *node;
+	SlonNode   *node;
 
 	rtcfg_lock();
 
@@ -327,8 +317,8 @@ rtcfg_storePath(int pa_server, char *pa_conninfo, int pa_connretry)
 
 		rtcfg_unlock();
 
-		slon_log(SLON_WARN, 
-				"storePath: unknown node ID %d - event pending\n", pa_server);
+		slon_log(SLON_WARN,
+			   "storePath: unknown node ID %d - event pending\n", pa_server);
 		rtcfg_storeNode(pa_server, "<event pending>");
 
 		rtcfg_lock();
@@ -339,12 +329,12 @@ rtcfg_storePath(int pa_server, char *pa_conninfo, int pa_connretry)
 	 * Store the (new) conninfo to the node
 	 */
 	slon_log(SLON_CONFIG,
-			"storePath: pa_server=%d pa_client=%d "
-			"pa_conninfo=\"%s\" pa_connretry=%d\n", 
-			pa_server, rtcfg_nodeid, pa_conninfo, pa_connretry);
+			 "storePath: pa_server=%d pa_client=%d "
+			 "pa_conninfo=\"%s\" pa_connretry=%d\n",
+			 pa_server, rtcfg_nodeid, pa_conninfo, pa_connretry);
 	if (node->pa_conninfo != NULL)
 		free(node->pa_conninfo);
-	node->pa_conninfo  = strdup(pa_conninfo);
+	node->pa_conninfo = strdup(pa_conninfo);
 	node->pa_connretry = pa_connretry;
 
 	rtcfg_unlock();
@@ -357,15 +347,14 @@ rtcfg_storePath(int pa_server, char *pa_conninfo, int pa_connretry)
 }
 
 
-/* ----------
- * rtcfg_dropPath
- * ----------
+/*
+ * ---------- rtcfg_dropPath ----------
  */
 void
 rtcfg_dropPath(int pa_server)
 {
-	SlonNode	   *node;
-	SlonListen	   *listen;
+	SlonNode   *node;
+	SlonListen *listen;
 
 	rtcfg_lock();
 
@@ -375,15 +364,14 @@ rtcfg_dropPath(int pa_server)
 
 		rtcfg_unlock();
 
-		slon_log(SLON_WARN, 
-				"dropPath: unknown node ID %d\n", pa_server);
+		slon_log(SLON_WARN,
+				 "dropPath: unknown node ID %d\n", pa_server);
 
 		return;
 	}
 
 	/*
-	 * Drop all listen information as well
-	 * at this provider. Without a path we
+	 * Drop all listen information as well at this provider. Without a path we
 	 * cannot listen.
 	 */
 	while (node->listen_head != NULL)
@@ -398,11 +386,11 @@ rtcfg_dropPath(int pa_server)
 	 * Remove the conninfo.
 	 */
 	slon_log(SLON_CONFIG,
-			"dropPath: pa_server=%d pa_client=%d\n",
-			pa_server, rtcfg_nodeid);
+			 "dropPath: pa_server=%d pa_client=%d\n",
+			 pa_server, rtcfg_nodeid);
 	if (node->pa_conninfo != NULL)
 		free(node->pa_conninfo);
-	node->pa_conninfo  = NULL;
+	node->pa_conninfo = NULL;
 	node->pa_connretry = 0;
 
 	rtcfg_unlock();
@@ -415,15 +403,78 @@ rtcfg_dropPath(int pa_server)
 }
 
 
-/* ----------
- * rtcfg_storeListen
- * ----------
+/*
+ * ---------- rtcfg_storeListen ----------
+ */
+void
+rtcfg_reloadListen(PGconn *db)
+{
+	SlonDString query;
+	PGresult   *res;
+	int			i	  ,
+				n;
+	SlonNode   *node;
+	SlonListen *listen;
+
+	/*
+	 * Clear out the entire Listen configuration
+	 */
+	rtcfg_lock();
+	for (node = rtcfg_node_list_head; node; node = node->next)
+	{
+		while ((listen = node->listen_head) != NULL)
+		{
+			DLLIST_REMOVE(node->listen_head, node->listen_tail, listen);
+			free(listen);
+		}
+	}
+	rtcfg_unlock();
+	rtcfg_seq_bump();
+
+	/*
+	 * Read configuration table sl_listen - the interesting pieces
+	 */
+	dstring_init(&query);
+
+	slon_mkquery(&query,
+				 "select li_origin, li_provider "
+				 "from %s.sl_listen where li_receiver = %d",
+				 rtcfg_namespace, rtcfg_nodeid);
+	res = PQexec(db, dstring_data(&query));
+	if (PQresultStatus(res) != PGRES_TUPLES_OK)
+	{
+		slon_log(SLON_FATAL, "Cannot get listen config - %s",
+				 PQresultErrorMessage(res));
+		PQclear(res);
+		dstring_free(&query);
+		slon_exit(-1);
+	}
+	for (i = 0, n = PQntuples(res); i < n; i++)
+	{
+		int			li_origin = (int)strtol(PQgetvalue(res, i, 0), NULL, 10);
+		int			li_provider = (int)strtol(PQgetvalue(res, i, 1), NULL, 10);
+
+		rtcfg_storeListen(li_origin, li_provider);
+	}
+	PQclear(res);
+
+	dstring_free(&query);
+
+	for (node = rtcfg_node_list_head; node; node = node->next)
+	{
+		rtcfg_startStopNodeThread(node);
+	}
+}
+
+
+/*
+ * ---------- rtcfg_storeListen ----------
  */
 void
 rtcfg_storeListen(int li_origin, int li_provider)
 {
-	SlonNode	   *node;
-	SlonListen	   *listen;
+	SlonNode   *node;
+	SlonListen *listen;
 
 	rtcfg_lock();
 
@@ -431,23 +482,23 @@ rtcfg_storeListen(int li_origin, int li_provider)
 	if (!node)
 	{
 		slon_log(SLON_FATAL,
-				"storeListen: unknown node ID %d\n", li_provider);
+				 "storeListen: unknown node ID %d\n", li_provider);
 		slon_abort();
 		return;
 	}
 
 	/*
-	 * Check if we already listen for events from that origin
-	 * at this provider.
+	 * Check if we already listen for events from that origin at this
+	 * provider.
 	 */
 	for (listen = node->listen_head; listen; listen = listen->next)
 	{
 		if (listen->li_origin == li_origin)
 		{
 			slon_log(SLON_DEBUG2,
-					"storeListen: li_origin=%d li_receiver=%d "
-					"li_provider=%d - already listening\n",
-					li_origin, rtcfg_nodeid, li_provider);
+					 "storeListen: li_origin=%d li_receiver=%d "
+					 "li_provider=%d - already listening\n",
+					 li_origin, rtcfg_nodeid, li_provider);
 			rtcfg_unlock();
 			return;
 		}
@@ -457,10 +508,10 @@ rtcfg_storeListen(int li_origin, int li_provider)
 	 * Add the new event origin to the provider (this node)
 	 */
 	slon_log(SLON_CONFIG,
-			"storeListen: li_origin=%d li_receiver=%d li_provider=%d\n",
-			li_origin, rtcfg_nodeid, li_provider);
+			 "storeListen: li_origin=%d li_receiver=%d li_provider=%d\n",
+			 li_origin, rtcfg_nodeid, li_provider);
 
-	listen = (SlonListen *)malloc(sizeof(SlonListen));
+	listen = (SlonListen *) malloc(sizeof(SlonListen));
 	if (listen == NULL)
 	{
 		perror("rtcfg_storeListen: malloc()");
@@ -481,15 +532,14 @@ rtcfg_storeListen(int li_origin, int li_provider)
 }
 
 
-/* ----------
- * rtcfg_dropListen
- * ----------
+/*
+ * ---------- rtcfg_dropListen ----------
  */
 void
 rtcfg_dropListen(int li_origin, int li_provider)
 {
-	SlonNode	   *node;
-	SlonListen	   *listen;
+	SlonNode   *node;
+	SlonListen *listen;
 
 	rtcfg_lock();
 
@@ -497,23 +547,22 @@ rtcfg_dropListen(int li_origin, int li_provider)
 	if (!node)
 	{
 		slon_log(SLON_FATAL,
-				"dropListen: unknown node ID %d\n", li_provider);
+				 "dropListen: unknown node ID %d\n", li_provider);
 		slon_abort();
 		return;
 	}
 
 	/*
-	 * Find that listen entry
-	 * at this provider.
+	 * Find that listen entry at this provider.
 	 */
 	for (listen = node->listen_head; listen; listen = listen->next)
 	{
 		if (listen->li_origin == li_origin)
 		{
 			slon_log(SLON_CONFIG,
-					"dropListen: li_origin=%d li_receiver=%d "
-					"li_provider=%d\n",
-					li_origin, rtcfg_nodeid, li_provider);
+					 "dropListen: li_origin=%d li_receiver=%d "
+					 "li_provider=%d\n",
+					 li_origin, rtcfg_nodeid, li_provider);
 
 			DLLIST_REMOVE(node->listen_head, node->listen_tail, listen);
 			free(listen);
@@ -528,22 +577,23 @@ rtcfg_dropListen(int li_origin, int li_provider)
 			return;
 		}
 	}
+
 	rtcfg_unlock();
 
 	/*
 	 * Add the new event origin to the provider (this node)
 	 */
 	slon_log(SLON_DEBUG1,
-			"storeListen: li_origin=%d li_receiver=%d li_provider=%d "
-			"- not listening\n",
-			li_origin, rtcfg_nodeid, li_provider);
+			 "storeListen: li_origin=%d li_receiver=%d li_provider=%d "
+			 "- not listening\n",
+			 li_origin, rtcfg_nodeid, li_provider);
 }
 
 
 void
 rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 {
-	SlonSet	   *set;
+	SlonSet    *set;
 
 	rtcfg_lock();
 
@@ -554,13 +604,13 @@ rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 	{
 		if (set->set_id == set_id)
 		{
-			int old_origin = set->set_origin;
+			int			old_origin = set->set_origin;
 
 			slon_log(SLON_CONFIG,
-					"storeSet: set_id=%d set_origin=%d "
-					"set_comment='%s' - update set\n",
-					set_id, set_origin, 
-					(set_comment == NULL) ? "<unchanged>" : set_comment);
+					 "storeSet: set_id=%d set_origin=%d "
+					 "set_comment='%s' - update set\n",
+					 set_id, set_origin,
+					 (set_comment == NULL) ? "<unchanged>" : set_comment);
 			set->set_origin = set_origin;
 			if (set_comment != NULL)
 			{
@@ -580,9 +630,9 @@ rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 	 * Add a new set to the configuration
 	 */
 	slon_log(SLON_CONFIG,
-			"storeSet: set_id=%d set_origin=%d set_comment='%s'\n",
-			set_id, set_origin, set_comment);
-	set = (SlonSet *)malloc(sizeof(SlonSet));
+			 "storeSet: set_id=%d set_origin=%d set_comment='%s'\n",
+			 set_id, set_origin, set_comment);
+	set = (SlonSet *) malloc(sizeof(SlonSet));
 	if (set == NULL)
 	{
 		perror("rtcfg_storeSet: malloc()");
@@ -606,7 +656,7 @@ rtcfg_storeSet(int set_id, int set_origin, char *set_comment)
 void
 rtcfg_dropSet(int set_id)
 {
-	SlonSet	   *set;
+	SlonSet    *set;
 
 	rtcfg_lock();
 
@@ -617,10 +667,10 @@ rtcfg_dropSet(int set_id)
 	{
 		if (set->set_id == set_id)
 		{
-			int old_origin = set->set_origin;
+			int			old_origin = set->set_origin;
 
 			slon_log(SLON_CONFIG,
-					"dropSet: set_id=%d\n", set_id);
+					 "dropSet: set_id=%d\n", set_id);
 			DLLIST_REMOVE(rtcfg_set_list_head, rtcfg_set_list_tail, set);
 			free(set->set_comment);
 			free(set);
@@ -633,14 +683,14 @@ rtcfg_dropSet(int set_id)
 	}
 
 	slon_log(SLON_CONFIG,
-			"dropSet: set_id=%d - set not found\n", set_id);
+			 "dropSet: set_id=%d - set not found\n", set_id);
 	rtcfg_unlock();
 }
 
 void
 rtcfg_moveSet(int set_id, int old_origin, int new_origin, int sub_provider)
 {
-	SlonSet	   *set;
+	SlonSet    *set;
 
 	rtcfg_lock();
 
@@ -652,10 +702,10 @@ rtcfg_moveSet(int set_id, int old_origin, int new_origin, int sub_provider)
 		if (set->set_id == set_id)
 		{
 			slon_log(SLON_CONFIG,
-					"moveSet: set_id=%d old_origin=%d "
-					"new_origin=%d\n",
-					set_id, old_origin, new_origin);
-				
+					 "moveSet: set_id=%d old_origin=%d "
+					 "new_origin=%d\n",
+					 set_id, old_origin, new_origin);
+
 			set->set_origin = new_origin;
 			set->sub_provider = sub_provider;
 			if (rtcfg_nodeid == old_origin)
@@ -688,7 +738,7 @@ rtcfg_moveSet(int set_id, int old_origin, int new_origin, int sub_provider)
 void
 rtcfg_storeSubscribe(int sub_set, int sub_provider, char *sub_forward)
 {
-	SlonSet	   *set;
+	SlonSet    *set;
 	int			old_provider = -1;
 
 	rtcfg_lock();
@@ -701,16 +751,17 @@ rtcfg_storeSubscribe(int sub_set, int sub_provider, char *sub_forward)
 		if (set->set_id == sub_set)
 		{
 			slon_log(SLON_CONFIG,
-					"storeSubscribe: sub_set=%d sub_provider=%d "
-					"sub_forward='%s'\n",
-					sub_set, sub_provider, sub_forward);
+					 "storeSubscribe: sub_set=%d sub_provider=%d "
+					 "sub_forward='%s'\n",
+					 sub_set, sub_provider, sub_forward);
 			old_provider = set->sub_provider;
 			if (set->sub_provider < 0)
 				set->sub_active = 0;
 			set->sub_provider = sub_provider;
-			set->sub_forward  = (*sub_forward == 't');
+			set->sub_forward = (*sub_forward == 't');
 			rtcfg_unlock();
 			rtcfg_seq_bump();
+
 			/*
 			 * Wakeup the worker threads for the old and new provider
 			 */
@@ -723,7 +774,7 @@ rtcfg_storeSubscribe(int sub_set, int sub_provider, char *sub_forward)
 	}
 
 	slon_log(SLON_FATAL,
-			"storeSubscribe: set %d not found\n", sub_set);
+			 "storeSubscribe: set %d not found\n", sub_set);
 	rtcfg_unlock();
 	slon_abort();
 }
@@ -732,7 +783,7 @@ rtcfg_storeSubscribe(int sub_set, int sub_provider, char *sub_forward)
 void
 rtcfg_enableSubscription(int sub_set, int sub_provider, char *sub_forward)
 {
-	SlonSet	   *set;
+	SlonSet    *set;
 	int			old_provider = -1;
 
 	rtcfg_lock();
@@ -745,11 +796,11 @@ rtcfg_enableSubscription(int sub_set, int sub_provider, char *sub_forward)
 		if (set->set_id == sub_set)
 		{
 			slon_log(SLON_CONFIG,
-					"enableSubscription: sub_set=%d\n",
-					sub_set);
+					 "enableSubscription: sub_set=%d\n",
+					 sub_set);
 			old_provider = set->sub_provider;
 			set->sub_provider = sub_provider;
-			set->sub_forward  = (*sub_forward == 't');
+			set->sub_forward = (*sub_forward == 't');
 			if (set->sub_provider >= 0)
 				set->sub_active = 1;
 			else
@@ -765,7 +816,7 @@ rtcfg_enableSubscription(int sub_set, int sub_provider, char *sub_forward)
 	}
 
 	slon_log(SLON_FATAL,
-			"enableSubscription: set %d not found\n", sub_set);
+			 "enableSubscription: set %d not found\n", sub_set);
 	rtcfg_unlock();
 	slon_abort();
 }
@@ -774,7 +825,7 @@ rtcfg_enableSubscription(int sub_set, int sub_provider, char *sub_forward)
 void
 rtcfg_unsubscribeSet(int sub_set)
 {
-	SlonSet	   *set;
+	SlonSet    *set;
 	int			old_provider = -1;
 
 	rtcfg_lock();
@@ -787,14 +838,15 @@ rtcfg_unsubscribeSet(int sub_set)
 		if (set->set_id == sub_set)
 		{
 			slon_log(SLON_CONFIG,
-					"unsubscribeSet: sub_set=%d\n",
-					sub_set);
+					 "unsubscribeSet: sub_set=%d\n",
+					 sub_set);
 			old_provider = set->sub_provider;
 			set->sub_provider = -1;
 			set->sub_active = false;
 			set->sub_forward = false;
 			rtcfg_unlock();
 			rtcfg_seq_bump();
+
 			/*
 			 * Wakeup the worker threads for the old and new provider
 			 */
@@ -805,21 +857,20 @@ rtcfg_unsubscribeSet(int sub_set)
 	}
 
 	slon_log(SLON_FATAL,
-			"unsubscribeSet: set %d not found\n", sub_set);
+			 "unsubscribeSet: set %d not found\n", sub_set);
 	rtcfg_unlock();
 	slon_abort();
 }
 
 
-/* ----------
- * rtcfg_startStopNodeThread
- * ----------
+/*
+ * ---------- rtcfg_startStopNodeThread ----------
  */
 static void
-rtcfg_startStopNodeThread(SlonNode *node)
+rtcfg_startStopNodeThread(SlonNode * node)
 {
-	int		need_listen = false;
-	int		need_wakeup = false;
+	int			need_listen = false;
+	int			need_wakeup = false;
 
 	rtcfg_lock();
 
@@ -832,23 +883,23 @@ rtcfg_startStopNodeThread(SlonNode *node)
 		{
 			case SLON_TSTAT_NONE:
 				if (pthread_create(&(node->worker_thread), NULL,
-						remoteWorkerThread_main, (void *)node) < 0)
+								   remoteWorkerThread_main, (void *)node) < 0)
 				{
 					slon_log(SLON_FATAL,
-							"startStopNodeThread: cannot create "
-							"remoteWorkerThread - %s\n",
-							strerror(errno));
+							 "startStopNodeThread: cannot create "
+							 "remoteWorkerThread - %s\n",
+							 strerror(errno));
 					rtcfg_unlock();
 					slon_abort();
 				}
 				node->worker_status = SLON_TSTAT_RUNNING;
 				break;
-			
+
 			case SLON_TSTAT_RUNNING:
 				break;
 
 			default:
-printf("TODO: ********** rtcfg_startStopNodeThread: restart node worker\n");
+				printf("TODO: ********** rtcfg_startStopNodeThread: restart node worker\n");
 		}
 	}
 	else
@@ -886,12 +937,12 @@ printf("TODO: ********** rtcfg_startStopNodeThread: restart node worker\n");
 			case SLON_TSTAT_NONE:
 				node->listen_status = SLON_TSTAT_RUNNING;
 				if (pthread_create(&(node->listen_thread), NULL,
-						remoteListenThread_main, (void *)node) < 0)
+								   remoteListenThread_main, (void *)node) < 0)
 				{
 					slon_log(SLON_FATAL,
-							"startStopNodeThread: cannot create "
-							"remoteListenThread - %s\n",
-							strerror(errno));
+							 "startStopNodeThread: cannot create "
+							 "remoteListenThread - %s\n",
+							 strerror(errno));
 					rtcfg_unlock();
 					slon_abort();
 				}
@@ -960,7 +1011,7 @@ printf("TODO: ********** rtcfg_startStopNodeThread: restart node worker\n");
 void
 rtcfg_needActivate(int no_id)
 {
-	struct to_activate	   *anode;
+	struct to_activate *anode;
 
 	anode = (struct to_activate *)malloc(sizeof(struct to_activate));
 	if (anode == NULL)
@@ -976,7 +1027,7 @@ rtcfg_needActivate(int no_id)
 void
 rtcfg_doActivate(void)
 {
-    while (to_activate_head != NULL)
+	while (to_activate_head != NULL)
 	{
 		struct to_activate *anode = to_activate_head;
 
@@ -1054,7 +1105,7 @@ rtcfg_seq_bump(void)
 int64
 rtcfg_seq_get(void)
 {
-	int64 retval;
+	int64		retval;
 
 	pthread_mutex_lock(&cfgseq_lock);
 	retval = cfgseq;
@@ -1065,8 +1116,8 @@ rtcfg_seq_get(void)
 
 /*
  * Local Variables:
- *  tab-width: 4
- *  c-indent-level: 4
- *  c-basic-offset: 4
+ *	tab-width: 4
+ *	c-indent-level: 4
+ *	c-basic-offset: 4
  * End:
  */
